class Output {

  constructor() {
    this.abapCode = `
***************************************************************
* This code was generated by openapi-json-to-abap on {{date}}.
* DO NOT MODIFY THIS CODE MANUALLY!
* {{title}}: v{{version}} 
***************************************************************

DATA: ls_meta TYPE /weco/ws_oai_meta,
ls_metapath TYPE /weco/ws_oai_metapath,
ls_metareqbody TYPE /weco/ws_oai_metareqbody,
ls_metaprop TYPE /weco/ws_oai_metaprop,
ls_metaparam TYPE /weco/ws_oai_metaparam.`;

    this.pathStatement =
      `CLEAR: ls_metapath.
ls_metapath-path = '{{path}}'.
ls_metapath-meth = '{{method}}'.
ls_metapath-oprid = '{{operationId}}'.
ls_metapath-struci = '{{wecoRequest}}'.
ls_metapath-secur = '{{security}}'.
ls_metapath-captc = '{{captcha}}'.
{{pathGuardStatement}}`;

    this.pathGuardStatement = `APPEND '{{guardId}}' TO ls_metapath-guard.\n`;

    this.parameterStatement =
      `CLEAR: ls_metaparam.
ls_metaparam-name = '{{name}}'.
ls_metaparam-parin = '{{inSegment}}'.
ls_metaparam-flreq = '{{required}}'.
ls_metaparam-type = '{{type}}'.{{parameterEnumStatement}}
APPEND ls_metaparam TO ls_metapath-param.\n`;

    this.parameterEnumStatement = `APPEND '{{enumValue}}' TO ls_metaparam-enum.\n`;

    this.requestBodyStatement =
      `CLEAR: ls_metareqbody.
ls_metareqbody-flreq = '{{required}}'.
ls_metareqbody-ctype = '{{contentType}}'.\n`;

    this.requestBodyPropertyStatement =
      `CLEAR: ls_metaprop.
ls_metaprop-name = '{{name}}'.
ls_metaprop-flreq = '{{required}}'.
ls_metaprop-type = '{{type}}'.{{requestBodyPropertyEnumStatement}}    
APPEND ls_metaprop TO ls_metareqbody-propr.\n`;

    this.requestBodyPropertyEnumStatement = `APPEND '{{enumValue}}' TO ls_metaprop-enum.\n`;
  }

  finishPath() {
    var s = `APPEND ls_metapath TO rs_meta-path.\n`;

    this.appendStatement(s);
  }

  finishRequestBody() {
    var s = `ls_metapath-reqbo = ls_metareqbody.\n`;
    this.appendStatement(s);
  }

  addMeta(version, title) {
    var s = this.abapCode;

    // Replace \n
    var v = version + "";
    v = v.replace('\n', '');
    var t = title + "";
    t = t.replace('\n', '');

    s = s.replace('{{version}}', v);
    s = s.replace('{{title}}', t);

    var date = new Date(Date.now()).toISOString();
    s = s.replace('{{date}}', date);
    this.abapCode = s;
  }

  addPath(path, method, operationId, wecoRequest, security, guards, captcha) {
    var s = this.pathStatement;
    s = s.replace('{{path}}', path);
    s = s.replace('{{method}}', method);
    s = s.replace('{{operationId}}', operationId);
    s = s.replace('{{wecoRequest}}', wecoRequest);
    s = s.replace('{{security}}', security === undefined ? '' : security);
    // weco/gp 250522 Captcha
    s = s.replace('{{captcha}}', captcha === undefined ? '' : captcha);

    // weco/gp 150222 Guards
    var guardTemplate = this.pathGuardStatement;
    var guardStatement = "";

    guards.forEach(value => {
      var g = guardTemplate.replace('{{guardId}}', value);
      guardStatement = guardStatement + g;
    });

    s = s.replace('{{pathGuardStatement}}', guardStatement);

    this.appendStatement(s);
  }

  addParameter(name, inSegment, required, type, enumValues) {
    var s = this.parameterStatement;
    s = s.replace('{{name}}', name);
    s = s.replace('{{inSegment}}', inSegment);
    s = s.replace('{{required}}', required ? 'X' : '');
    s = s.replace('{{type}}', type);

    var esTemplate = this.parameterEnumStatement;
    var enumStatement = "";
    enumValues.forEach(value => {
      var e = esTemplate.replace('{{enumValue}}', value);
      enumStatement = enumStatement + e;
    })

    if(enumStatement !== "") {
      enumStatement = "\n" + enumStatement;
    }

    s = s.replace('{{parameterEnumStatement}}', enumStatement);
    this.appendStatement(s);
  }

  addRequestBody(required, contentType) {
    var s = this.requestBodyStatement;
    s = s.replace('{{required}}', required ? 'X' : '');
    s = s.replace('{{contentType}}', contentType);
    this.appendStatement(s);
  }

  addRequestBodyProperty(name, type, required, enumValues) {
    var s = this.requestBodyPropertyStatement;
    s = s.replace('{{name}}', name);
    s = s.replace('{{type}}', type);
    s = s.replace('{{required}}', required ? 'X' : '');

    var esTemplate = this.requestBodyPropertyEnumStatement;
    var enumStatement = "";
    enumValues.forEach(value => {
      var e = esTemplate.replace('{{enumValue}}', value);
      enumStatement = enumStatement + e;
    })

    if(enumStatement !== "") {
      enumStatement = "\n" + enumStatement;
    }

    s = s.replace('{{requestBodyPropertyEnumStatement}}', enumStatement);

    this.appendStatement(s);
  }

  appendStatement(statement) {
    this.abapCode += statement;
  }

  getAbapCode() {
    var finalCode = `method GET_META_RAW.
    `
      + this.abapCode +
      `\nendmethod.`;

    return finalCode;
  }

}

module.exports = Output;